import * as React from 'react';
import Refractor from 'react-refractor/all';
import clsx from 'clsx';

type PrismDerivedElement = HTMLPreElement;

export interface PrismProps extends Omit<React.HTMLProps<PrismDerivedElement>, 'children'> {
  code?: string;
  language?: string;
  lineNumbers?: boolean;
  maxLineNumber?: number;
  tabSize?: number;
}

export const Prism = React.forwardRef<PrismDerivedElement, PrismProps>(({
  code,
  language = 'plain',
  lineNumbers = false,
  maxLineNumber = Infinity,
  tabSize = 2,
  className,
  style,
  ...etcProps
}, forwardedRef) => {
  const [highlightedCode, setHighlightedCode] = React.useState<string>('');

  if (!code) {
    return null;
  }

  return (
    <div
      className={clsx(
        'react-prism',
        className,
      )}
      style={{
        ...(style ?? {}),

      }}
    >
      {lineNumbers && (
        <pre
          style={{
            padding: 0,
            overflow: 'visible',
            margin: 0,
            lineHeight: 1.2,
            textAlign: 'right',
          }}
        >
          <code>
            {new Array(maxLineNumber).fill(0).map((_, i) => (
              <React.Fragment key={i}>
                {i > 0 && '\n'}
                {i + 1}
              </React.Fragment>
            ))}
          </code>
        </pre>
      )}
      <Refractor
        {...etcProps}
        language={language}
        value={code}
        className="highlight"
      />
    </div>
  )
});

Prism.displayName = 'Prism';

